#include "generalized.h"

/*
 * generalized.c
 *
 * this file contains generalized functions of use in various parts of
 * mdebs.
 */

/*
 * acctQuery()
 *
 * Given an account number, returns 1 if the account exists, 0 otherwise.
 * Bails out on error.
 */
char *acctQuery(char *nam, char *acct, char *anal)
{
  char miscbuff[81];
  int resultcode, noofrecs;
  
  sprintf(miscbuff, "SELECT * FROM osnova WHERE acct='%s' and anal='%s';", acct, anal);
  resultcode = runQuery(miscbuff);
  if (resultcode == 3) {                /* this is the proper result code */
    noofrecs = numberReturned();
    if (noofrecs > 1) {
      fprintf(stderr, "Error: Corrupted database: Account number must be unique.\n");
      exit_nicely();
    }
  } else {
    fprintf(stderr, "Fatal Error: Query returned improper result code (%d).\n", resultcode);
    exit_nicely();
  }
  if (noofrecs == 0) return NULL;
  else {
    return (nam = getField("desig"));
  }
}

/*
 * transQuery()
 *
 * Given a transaction, determines if it exists in the database
 * Bails out on error, returns 0 if not found, 1 otherwise.
 */
int transQuery(int datum, int porc)
{
  char miscbuff[81];
  int resultcode, noofrecs;
  char *pgDatum;

  pgDatum = malloc(11);
  pgDatum = pgConvertDate(pgDatum, datum);

  sprintf(miscbuff, "SELECT * FROM descs WHERE ent_date='%s' and por_cislo=%d;", pgDatum, porc);
  resultcode = runQuery(miscbuff);
  if (resultcode == 3) {                /* this is the proper result code */
    noofrecs = numberReturned();
    if (noofrecs > 1) {
      fprintf(stderr, "Error: Corrupted database: Account number must be unique.\n");
      exit_nicely();
    } else
      if (noofrecs == 1) {
	return 1;
      }
  } else {
    fprintf(stderr, "Error: Query returned improper result code (%d).\n", resultcode);
    exit_nicely();
  }
  return noofrecs;
}

/*
 * NathanConvertDate()
 *
 * This function converts dates from a 'mm-dd-yyyy' character string to
 * an yyyymmdd integer.
 *
 * This is a "garbage in, garbage out" function.
 */
int NathanConvertDate(char *pgdate)
{
  short year, month, day;
  char buf[4];

  buf[0] = *pgdate;
  buf[1] = *(pgdate + 1);
  buf[2] = '\0';
  month = atoi(buf);
  
  buf[0] = *(pgdate + 3);
  buf[1] = *(pgdate + 4);
  buf[2] = '\0';
  day = atoi(buf);

  buf[0] = *(pgdate + 6);
  buf[1] = *(pgdate + 7);
  buf[2] = *(pgdate + 8);
  buf[3] = *(pgdate + 9);
  buf[4] = '\0';
  year = atoi(buf);

  return ((year*10000)+(month*100)+day);
}

/*
 * pgConvertDate()
 *
 * This function is the reverse of the NathanConvertDate()
 * function. It converts dates from a yyyymmdd integer to a
 * 'mm-dd-yyyy' character string.
 *
 * This is a "garbage in, garbage out" function.  
 */
char *pgConvertDate(char *pgdate, int nathandate)
{
  short year, month, day;

  year = nathandate / 10000;
  nathandate = nathandate - (year*10000);
  month = nathandate / 100;
  nathandate = nathandate - (month*100);
  day = nathandate;

  sprintf(pgdate, "%02d-%02d-%4d", month, day, year);
  return pgdate;
}

/*
 * countNonWhiteSpace()
 *
 * Counts the number of non-white-space characters in a particular string
 */
int countNonWhiteSpace(char *buff)
{
  int i, numnonwhite;
  i = 0; numnonwhite = 0;
  while (*(buff+i) != '\0') {
      if ((*(buff+i) != ' ') &&
	  (*(buff+i) != '\t') &&
	  (*(buff+i) != '\n')) {
	numnonwhite++;
      }
      i++;
  }
  return numnonwhite;
}

/*
 * valiDate()
 *
 * Checks an integer date to see if it is valid
 * (still quite rudimentary)
 */
int valiDate(int datum)
{
  struct tm intDatum;
  time_t intrep;
  char *yyyymmdd;
  int ymd;
  int year, month, day;

  /*
   * is the given date before 1900?
   */
  if ((datum - 19000000) < 0) {
    return 0;
  }

  /*
   * Initialize the broken down time structure
   */
  intDatum.tm_sec = 0;
  intDatum.tm_min = 0;
  intDatum.tm_hour = 12;
  day = datum % 100;
  intDatum.tm_mday = day;
  month = (datum % 10000) / 100 - 1;
  intDatum.tm_mon = month;
  year = (datum - 19000000) / 10000;
  intDatum.tm_year = year;
  intDatum.tm_isdst = -1;

  /*
   * Get the calendar time expressed in seconds relative to Jan. 1, 1970
   */
  intrep = mktime(&intDatum);

  /*
   * If the call to mktime failed, then the date is invalid
   */
  if (intrep == -1) {
    return 0;
  }

  /*
   * Now construct a YYYYMMDD string with the time returned by mktim()
   */
  yyyymmdd = malloc(9);
  strftime(yyyymmdd, 9, "%Y%m%d", &intDatum);

  /*
   * Convert the string to an integer
   */
  ymd = atoi(yyyymmdd);

  /*
   * Compare ymd with the date in question -- if they are the same, then
   * the date is valid.  If they are different, then the date is invalid.
   */
  if (ymd != datum) {
    return 0;
  }
  return 1;
}

/*
 * This function returns a string containing the current date and time
 * Useful for logging, I think.
 */
char *theTimeIs(void)
{
  time_t now;
  char *output;
  now = time(NULL);
  output = ctime(&now);
  *(output+24) = '\0';
  return output;
}

/*
 * queryFiscalYear()
 *
 * Function to determine the fiscal year of the current database
 */
int queryFiscalYear(int *startd, int *endd)
{
  char miscbuff[81];
  int resultcode, noofrecs;

  sprintf(miscbuff, "SELECT * FROM fiscyear;");
  resultcode = runQuery(miscbuff);
  if (resultcode == 3) {                /* this is the proper result code */
    noofrecs = numberReturned();
    if (noofrecs > 1) {
      fprintf(stderr, "Error: Corrupted database: fiscyear may contain a maximum of one record.\n");
      exit_nicely();
    } else
      if (noofrecs == 1) {
	/* code to get the fields from the one record returned */
	*(startd) = NathanConvertDate(getField("startd"));
	*(endd) = NathanConvertDate(getField("endd"));
      } else {
        fprintf(stderr, "Error: fiscyear table not initialized.\n");
        exit_nicely();
      }
  } else {
    fprintf(stderr, "Error: Query returned improper result code (%d).\n", resultcode);
    exit_nicely();
  }
  return 1;
}

